@@ -15,6 +15,7 @@
 
 #include "lldb/lldb-private.h"
 #include "lldb/Host/Mutex.h"
+#include "lldb/Utility/Iterable.h"
 
 namespace lldb_private {
 
@@ -137,7 +138,7 @@ public:
     ///
     /// Clears the list of modules and releases a reference to each
     /// module object and if the reference count goes to zero, the
-    /// module will be deleted. Also relese all memory that might be
+    /// module will be deleted. Also release all memory that might be
     /// held by any collection classes (like std::vector)
     //------------------------------------------------------------------
     void
@@ -149,7 +150,7 @@ public:
     /// the supplied stream \a s.
     ///
     /// @param[in] s
-    ///     The stream to which to dump the object descripton.
+    ///     The stream to which to dump the object description.
     ///
     /// @see Module::Dump(Stream *) const
     //------------------------------------------------------------------
@@ -277,6 +278,16 @@ public:
                          uint32_t name_type_mask,
                          SymbolContextList& sc_list);
 
+    //------------------------------------------------------------------
+    /// @see Module::FindFunctions ()
+    //------------------------------------------------------------------
+    size_t
+        FindFunctions(const RegularExpression &name,
+        bool include_symbols,
+        bool include_inlines,
+        bool append,
+        SymbolContextList& sc_list);
+
     //------------------------------------------------------------------
     /// Find global and static variables by name.
     ///
@@ -307,7 +318,7 @@ public:
                          VariableList& variable_list) const;
 
     //------------------------------------------------------------------
-    /// Find global and static variables by regular exression.
+    /// Find global and static variables by regular expression.
     ///
     /// @param[in] regex
     ///     A regular expression to use when matching the name.
@@ -577,6 +588,21 @@ protected:
 
     Notifier* m_notifier;
     
+public:
+    typedef LockingAdaptedIterable<collection, lldb::ModuleSP, vector_adapter> ModuleIterable;
+    ModuleIterable
+    Modules()
+    {
+        return ModuleIterable(m_modules, GetMutex());
+    }
+    
+    typedef AdaptedIterable<collection, lldb::ModuleSP, vector_adapter> ModuleIterableNoLocking;
+    ModuleIterableNoLocking
+    ModulesNoLocking ()
+    {
+        return ModuleIterableNoLocking(m_modules);
+    }
+    
 };
 
 } // namespace lldb_private
