@@ -24,8 +24,6 @@
 #include "lldb/lldb-enumerations.h"
 
 #include "lldb/Core/ValueObject.h"
-#include "lldb/Interpreter/ScriptInterpreterPython.h"
-#include "lldb/Symbol/Type.h"
 
 namespace lldb_private {
     class SyntheticChildrenFrontEnd
@@ -81,14 +79,71 @@ namespace lldb_private {
         virtual bool
         MightHaveChildren () = 0;
         
+        // if this function returns a non-null ValueObject, then the returned ValueObject will stand
+        // for this ValueObject whenever a "value" request is made to this ValueObject
+        virtual lldb::ValueObjectSP
+        GetSyntheticValue () { return nullptr; }
+        
         typedef std::shared_ptr<SyntheticChildrenFrontEnd> SharedPointer;
         typedef std::unique_ptr<SyntheticChildrenFrontEnd> AutoPointer;
         
+    protected:
+        lldb::ValueObjectSP
+        CreateValueObjectFromExpression (const char* name,
+                                         const char* expression,
+                                         const ExecutionContext& exe_ctx);
+        
+        lldb::ValueObjectSP
+        CreateValueObjectFromAddress (const char* name,
+                                      uint64_t address,
+                                      const ExecutionContext& exe_ctx,
+                                      ClangASTType type);
+        
+        lldb::ValueObjectSP
+        CreateValueObjectFromData (const char* name,
+                                   const DataExtractor& data,
+                                   const ExecutionContext& exe_ctx,
+                                   ClangASTType type);
+        
     private:
         bool m_valid;
         DISALLOW_COPY_AND_ASSIGN(SyntheticChildrenFrontEnd);
     };
     
+    class SyntheticValueProviderFrontEnd : public SyntheticChildrenFrontEnd
+    {
+    public:
+        SyntheticValueProviderFrontEnd (ValueObject &backend) :
+        SyntheticChildrenFrontEnd(backend)
+        {}
+        
+        virtual
+        ~SyntheticValueProviderFrontEnd ()
+        {
+        }
+        
+        virtual size_t
+        CalculateNumChildren () { return 0; }
+        
+        virtual lldb::ValueObjectSP
+        GetChildAtIndex (size_t idx) { return nullptr; }
+        
+        virtual size_t
+        GetIndexOfChildWithName (const ConstString &name) { return UINT32_MAX; }
+        
+        virtual bool
+        Update () { return false; }
+        
+        virtual bool
+        MightHaveChildren () { return false; }
+        
+        virtual lldb::ValueObjectSP
+        GetSyntheticValue () = 0;
+        
+    private:
+        DISALLOW_COPY_AND_ASSIGN(SyntheticValueProviderFrontEnd);
+    };
+    
     class SyntheticChildren
     {
     public:
@@ -328,37 +383,10 @@ namespace lldb_private {
         }
         
         void
-        AddExpressionPath (const std::string& path)
-        {
-            bool need_add_dot = true;
-            if (path[0] == '.' ||
-                (path[0] == '-' && path[1] == '>') ||
-                path[0] == '[')
-                need_add_dot = false;
-            // add a '.' symbol to help forgetful users
-            if(!need_add_dot)
-                m_expression_paths.push_back(path);
-            else
-                m_expression_paths.push_back(std::string(".") + path);
-        }
+        AddExpressionPath (const std::string& path);
         
         bool
-        SetExpressionPathAtIndex (size_t i, const std::string& path)
-        {
-            if (i >= GetCount())
-                return false;
-            bool need_add_dot = true;
-            if (path[0] == '.' ||
-                (path[0] == '-' && path[1] == '>') ||
-                path[0] == '[')
-                need_add_dot = false;
-            // add a '.' symbol to help forgetful users
-            if(!need_add_dot)
-                m_expression_paths[i] = path;
-            else
-                m_expression_paths[i] = std::string(".") + path;
-            return true;
-        }
+        SetExpressionPathAtIndex (size_t i, const std::string& path);
         
         bool
         IsScripted ()
@@ -410,24 +438,7 @@ namespace lldb_private {
             }
             
             virtual size_t
-            GetIndexOfChildWithName (const ConstString &name)
-            {
-                const char* name_cstr = name.GetCString();
-                for (size_t i = 0; i < filter->GetCount(); i++)
-                {
-                    const char* expr_cstr = filter->GetExpressionPathAtIndex(i);
-                    if (expr_cstr)
-                    {
-                        if (*expr_cstr == '.')
-                            expr_cstr++;
-                        else if (*expr_cstr == '-' && *(expr_cstr+1) == '>')
-                            expr_cstr += 2;
-                    }
-                    if (!::strcmp(name_cstr, expr_cstr))
-                        return i;
-                }
-                return UINT32_MAX;
-            }
+            GetIndexOfChildWithName (const ConstString &name);
             
             typedef std::shared_ptr<SyntheticChildrenFrontEnd> SharedPointer;
             
@@ -548,50 +559,28 @@ namespace lldb_private {
                       ValueObject &backend);
             
             bool
-            IsValid ()
-            {
-                return m_wrapper_sp.get() != nullptr && m_wrapper_sp->operator bool() && m_interpreter != nullptr;
-            }
+            IsValid ();
             
             virtual
             ~FrontEnd ();
             
             virtual size_t
-            CalculateNumChildren ()
-            {
-                if (!m_wrapper_sp || m_interpreter == NULL)
-                    return 0;
-                return m_interpreter->CalculateNumChildren(m_wrapper_sp);
-            }
+            CalculateNumChildren ();
             
             virtual lldb::ValueObjectSP
             GetChildAtIndex (size_t idx);
             
             virtual bool
-            Update ()
-            {
-                if (!m_wrapper_sp || m_interpreter == NULL)
-                    return false;
-                
-                return m_interpreter->UpdateSynthProviderInstance(m_wrapper_sp);
-            }
+            Update ();
             
             virtual bool
-            MightHaveChildren ()
-            {
-                if (!m_wrapper_sp || m_interpreter == NULL)
-                    return false;
-                
-                return m_interpreter->MightHaveChildrenSynthProviderInstance(m_wrapper_sp);
-            }
+            MightHaveChildren ();
             
             virtual size_t
-            GetIndexOfChildWithName (const ConstString &name)
-            {
-                if (!m_wrapper_sp || m_interpreter == NULL)
-                    return UINT32_MAX;
-                return m_interpreter->GetIndexOfChildWithName(m_wrapper_sp, name.GetCString());
-            }
+            GetIndexOfChildWithName (const ConstString &name);
+            
+            virtual lldb::ValueObjectSP
+            GetSyntheticValue ();
             
             typedef std::shared_ptr<SyntheticChildrenFrontEnd> SharedPointer;
             
