@@ -49,9 +49,13 @@ namespace lldb
     typedef void*               mutex_t;
     typedef void*               condition_t;
     typedef void*               rwlock_t;
-    typedef uintptr_t           thread_t;                   // Host thread type
+    typedef void*               process_t;                  // Process type is HANDLE
+    typedef void*               thread_t;                   // Host thread type
+    typedef void*               file_t;                     // Host file type
+    typedef void*               pipe_t;                     // Host pipe type
+    typedef unsigned int __w64  socket_t;                   // Host socket type
     typedef uint32_t            thread_key_t;
-    typedef void *              thread_arg_t;               // Host thread argument type
+    typedef void*               thread_arg_t;               // Host thread argument type
     typedef unsigned            thread_result_t;            // Host thread result type
     typedef thread_result_t     (*thread_func_t)(void *);   // Host thread function type
 }
@@ -68,7 +72,11 @@ namespace lldb
     typedef ::pthread_mutex_t   mutex_t;
     typedef pthread_cond_t      condition_t;
     typedef pthread_rwlock_t    rwlock_t;
+    typedef uint64_t            process_t;                  // Process type is just a pid.
     typedef pthread_t           thread_t;                   // Host thread type
+    typedef int                 file_t;                     // Host file type
+    typedef int                 pipe_t;                     // Host pipe type
+    typedef int                 socket_t;                   // Host socket type
     typedef pthread_key_t       thread_key_t;
     typedef void *              thread_arg_t;               // Host thread argument type
     typedef void *              thread_result_t;            // Host thread result type
@@ -81,9 +89,13 @@ namespace lldb
 {
     typedef void                (*LogOutputCallback) (const char *, void *baton);
     typedef bool                (*CommandOverrideCallback)(void *baton, const char **argv);
+    typedef bool                (*CommandOverrideCallbackWithResult)(void *baton,
+                                                                     const char **argv,
+                                                                     lldb_private::CommandReturnObject &result);
     typedef bool                (*ExpressionCancelCallback) (ExpressionEvaluationPhase phase, void *baton);
 }
 
+#define LLDB_INVALID_PROCESS             ((lldb::process_t)-1)
 #define LLDB_INVALID_HOST_THREAD         ((lldb::thread_t)NULL)
 #define IS_VALID_LLDB_HOST_THREAD(t)     ((t) != LLDB_INVALID_HOST_THREAD)
 
