@@ -233,7 +233,7 @@ ValueObjectMemory::UpdateValue ()
                 }
             }
 
-            if (GetClangType().IsAggregateType())
+            if (!CanProvideValue())
             {
                 // this value object represents an aggregate type whose
                 // children have values, but this object does not. So we
