@@ -649,18 +649,18 @@ ABISysV_x86_64::GetReturnValueObjectSimple (Thread &thread,
         return return_valobj_sp;
     
     const uint32_t type_flags = return_clang_type.GetTypeInfo ();
-    if (type_flags & ClangASTType::eTypeIsScalar)
+    if (type_flags & eTypeIsScalar)
     {
         value.SetValueType(Value::eValueTypeScalar);
 
         bool success = false;
-        if (type_flags & ClangASTType::eTypeIsInteger)
+        if (type_flags & eTypeIsInteger)
         {
             // Extract the register context so we can read arguments from registers
             
             const size_t byte_size = return_clang_type.GetByteSize();
             uint64_t raw_value = thread.GetRegisterContext()->ReadRegisterAsUnsigned(reg_ctx->GetRegisterInfoByName("rax", 0), 0);
-            const bool is_signed = (type_flags & ClangASTType::eTypeIsSigned) != 0;
+            const bool is_signed = (type_flags & eTypeIsSigned) != 0;
             switch (byte_size)
             {
             default:
@@ -699,9 +699,9 @@ ABISysV_x86_64::GetReturnValueObjectSimple (Thread &thread,
                 break;
             }
         }
-        else if (type_flags & ClangASTType::eTypeIsFloat)
+        else if (type_flags & eTypeIsFloat)
         {
-            if (type_flags & ClangASTType::eTypeIsComplex)
+            if (type_flags & eTypeIsComplex)
             {
                 // Don't handle complex yet.
             }
@@ -744,7 +744,7 @@ ABISysV_x86_64::GetReturnValueObjectSimple (Thread &thread,
                                                                ConstString(""));
 
     }
-    else if (type_flags & ClangASTType::eTypeIsPointer)
+    else if (type_flags & eTypeIsPointer)
     {
         unsigned rax_id = reg_ctx->GetRegisterInfoByName("rax", 0)->kinds[eRegisterKindLLDB];
         value.GetScalar() = (uint64_t)thread.GetRegisterContext()->ReadRegisterAsUnsigned(rax_id, 0);
@@ -753,7 +753,7 @@ ABISysV_x86_64::GetReturnValueObjectSimple (Thread &thread,
                                                            value,
                                                            ConstString(""));
     }
-    else if (type_flags & ClangASTType::eTypeIsVector)
+    else if (type_flags & eTypeIsVector)
     {
         const size_t byte_size = return_clang_type.GetByteSize();
         if (byte_size > 0)
