@@ -23,7 +23,7 @@ public:
     //------------------------------------------------------------
     // Class functions
     //------------------------------------------------------------
-    static lldb_private::Platform* 
+    static lldb::PlatformSP
     CreateInstance (bool force, const lldb_private::ArchSpec *arch);
 
     static void
@@ -49,27 +49,27 @@ public:
     //------------------------------------------------------------
     // lldb_private::PluginInterface functions
     //------------------------------------------------------------
-    virtual lldb_private::ConstString
-    GetPluginName()
+    lldb_private::ConstString
+    GetPluginName() override
     {
         return GetPluginNameStatic (IsHost());
     }
     
-    virtual uint32_t
-    GetPluginVersion()
+    uint32_t
+    GetPluginVersion() override
     {
         return 1;
     }
     
-    virtual lldb_private::Error
+    lldb_private::Error
     GetSharedModule (const lldb_private::ModuleSpec &module_spec,
                      lldb::ModuleSP &module_sp,
                      const lldb_private::FileSpecList *module_search_paths_ptr,
                      lldb::ModuleSP *old_module_sp_ptr,
-                     bool *did_create_ptr);
+                     bool *did_create_ptr) override;
     
-    virtual const char *
-    GetDescription ()
+    const char *
+    GetDescription () override
     {
         return GetDescriptionStatic (IsHost());
     }
@@ -79,24 +79,30 @@ public:
                    const lldb_private::UUID *uuid_ptr,
                    lldb_private::FileSpec &local_file);
     
-    virtual lldb_private::Error
+    lldb_private::Error
     GetFile (const lldb_private::FileSpec& source,
-             const lldb_private::FileSpec& destination)
+             const lldb_private::FileSpec& destination) override
     {
         return PlatformDarwin::GetFile (source,destination);
     }
     
-    virtual lldb_private::Error
+    lldb_private::Error
     GetFileWithUUID (const lldb_private::FileSpec &platform_file, 
                      const lldb_private::UUID *uuid_ptr,
-                     lldb_private::FileSpec &local_file);
+                     lldb_private::FileSpec &local_file) override;
     
     virtual bool
     GetSupportedArchitectureAtIndex (uint32_t idx, 
-                                     lldb_private::ArchSpec &arch);
+                                     lldb_private::ArchSpec &arch) override;
 
     virtual lldb_private::ConstString
-    GetSDKDirectory (lldb_private::Target &target);
+    GetSDKDirectory (lldb_private::Target &target) override;
+    
+    void
+    AddClangModuleCompilationOptions (std::vector<std::string> &options) override
+    {
+        return PlatformDarwin::AddClangModuleCompilationOptionsForSDKType(options, PlatformDarwin::SDKType::MacOSX);
+    }
 
 private:
     DISALLOW_COPY_AND_ASSIGN (PlatformMacOSX);
