@@ -37,7 +37,7 @@ public:
     //------------------------------------------------------------
     // lldb_private::PluginInterface functions
     //------------------------------------------------------------
-    static lldb_private::Platform*
+    static lldb::PlatformSP
     CreateInstance (bool force, const lldb_private::ArchSpec *arch);
 
 
@@ -60,8 +60,7 @@ public:
     // lldb_private::Platform functions
     //------------------------------------------------------------
     virtual Error
-    ResolveExecutable(const FileSpec &exe_file,
-                      const ArchSpec &arch,
+    ResolveExecutable(const lldb_private::ModuleSpec &module_spec,
                       lldb::ModuleSP &module_sp,
                       const FileSpecList *module_search_paths_ptr);
 
@@ -121,7 +120,6 @@ public:
     Attach(lldb_private::ProcessAttachInfo &attach_info,
            lldb_private::Debugger &debugger,
            lldb_private::Target *target,
-           lldb_private::Listener &listener,
            lldb_private::Error &error);
 
     virtual lldb_private::Error
