@@ -146,9 +146,17 @@ DWARFCallFrameInfo::ParseCIE (const dw_offset_t cie_offset)
     lldb::offset_t offset = cie_offset;
     if (m_cfi_data_initialized == false)
         GetCFIData();
-    const uint32_t length = m_cfi_data.GetU32(&offset);
-    const dw_offset_t cie_id = m_cfi_data.GetU32(&offset);
-    const dw_offset_t end_offset = cie_offset + length + 4;
+    uint32_t length = m_cfi_data.GetU32(&offset);
+    dw_offset_t cie_id, end_offset;
+    bool is_64bit = (length == UINT32_MAX);
+    if (is_64bit) {
+        length = m_cfi_data.GetU64(&offset);
+        cie_id = m_cfi_data.GetU64(&offset);
+        end_offset = cie_offset + length + 12;
+    } else {
+        cie_id = m_cfi_data.GetU32(&offset);
+        end_offset = cie_offset + length + 4;
+    }
     if (length > 0 && ((!m_is_eh_frame && cie_id == UINT32_MAX) || (m_is_eh_frame && cie_id == 0ul)))
     {
         size_t i;
@@ -210,20 +218,27 @@ DWARFCallFrameInfo::ParseCIE (const dw_offset_t cie_offset)
                             // FDE, which is the address of a language-specific
                             // data area (LSDA). The size of the LSDA pointer is
                             // specified by the pointer encoding used.
-                            m_cfi_data.GetU8(&offset);
+                            cie_sp->lsda_addr_encoding = m_cfi_data.GetU8(&offset);
                             break;
 
                         case 'P':
                             // Indicates the presence of two arguments in the
-                            // Augmentation Data of the cie_sp-> The first argument
+                            // Augmentation Data of the CIE. The first argument
                             // is 1-byte and represents the pointer encoding
                             // used for the second argument, which is the
                             // address of a personality routine handler. The
                             // size of the personality routine pointer is
                             // specified by the pointer encoding used.
+                            //
+                            // The address of the personality function will
+                            // be stored at this location.  Pre-execution, it
+                            // will be all zero's so don't read it until we're
+                            // trying to do an unwind & the reloc has been
+                            // resolved.
                         {
                             uint8_t arg_ptr_encoding = m_cfi_data.GetU8(&offset);
-                            m_cfi_data.GetGNUEHPointer(&offset, arg_ptr_encoding, LLDB_INVALID_ADDRESS, LLDB_INVALID_ADDRESS, LLDB_INVALID_ADDRESS);
+                            const lldb::addr_t pc_rel_addr = m_section_sp->GetFileAddress();
+                            cie_sp->personality_loc = m_cfi_data.GetGNUEHPointer(&offset, arg_ptr_encoding, pc_rel_addr, LLDB_INVALID_ADDRESS, LLDB_INVALID_ADDRESS);
                         }
                             break;
 
@@ -337,9 +352,19 @@ DWARFCallFrameInfo::GetFDEIndex ()
     while (m_cfi_data.ValidOffsetForDataOfSize (offset, 8))
     {
         const dw_offset_t current_entry = offset;
+        dw_offset_t cie_id, next_entry, cie_offset;
         uint32_t len = m_cfi_data.GetU32 (&offset);
-        dw_offset_t next_entry = current_entry + len + 4;
-        dw_offset_t cie_id = m_cfi_data.GetU32 (&offset);
+        bool is_64bit = (len == UINT32_MAX);
+        if (is_64bit) {
+            len = m_cfi_data.GetU64 (&offset);
+            cie_id = m_cfi_data.GetU64 (&offset);
+            next_entry = current_entry + len + 12;
+            cie_offset = current_entry + 12 - cie_id;
+        } else {
+            cie_id = m_cfi_data.GetU32 (&offset);
+            next_entry = current_entry + len + 4;
+            cie_offset = current_entry + 4 - cie_id;
+        }
 
         if (cie_id == 0 || cie_id == UINT32_MAX || len == 0)
         {
@@ -348,7 +373,6 @@ DWARFCallFrameInfo::GetFDEIndex ()
             continue;
         }
 
-        const dw_offset_t cie_offset = current_entry + 4 - cie_id;
         const CIE *cie = GetCIE (cie_offset);
         if (cie)
         {
@@ -388,7 +412,14 @@ DWARFCallFrameInfo::FDEToUnwindPlan (dw_offset_t dwarf_offset, Address startaddr
         GetCFIData();
 
     uint32_t length = m_cfi_data.GetU32 (&offset);
-    dw_offset_t cie_offset = m_cfi_data.GetU32 (&offset);
+    dw_offset_t cie_offset;
+    bool is_64bit = (length == UINT32_MAX);
+    if (is_64bit) {
+        length = m_cfi_data.GetU64 (&offset);
+        cie_offset = m_cfi_data.GetU64 (&offset);
+    } else {
+        cie_offset = m_cfi_data.GetU32 (&offset);
+    }
 
     assert (cie_offset != 0 && cie_offset != UINT32_MAX);
 
@@ -398,7 +429,7 @@ DWARFCallFrameInfo::FDEToUnwindPlan (dw_offset_t dwarf_offset, Address startaddr
     if (m_is_eh_frame)
     {
         unwind_plan.SetSourceName ("eh_frame CFI");
-        cie_offset = current_entry + 4 - cie_offset;
+        cie_offset = current_entry + (is_64bit ? 12 : 4) - cie_offset;
         unwind_plan.SetUnwindPlanValidAtAllInstructions (eLazyBoolNo);
     }
     else
@@ -415,7 +446,7 @@ DWARFCallFrameInfo::FDEToUnwindPlan (dw_offset_t dwarf_offset, Address startaddr
     const CIE *cie = GetCIE (cie_offset);
     assert (cie != nullptr);
 
-    const dw_offset_t end_offset = current_entry + length + 4;
+    const dw_offset_t end_offset = current_entry + length + (is_64bit ? 12 : 4);
 
     const lldb::addr_t pc_rel_addr = m_section_sp->GetFileAddress();
     const lldb::addr_t text_addr = LLDB_INVALID_ADDRESS;
@@ -425,11 +456,39 @@ DWARFCallFrameInfo::FDEToUnwindPlan (dw_offset_t dwarf_offset, Address startaddr
     AddressRange range (range_base, m_objfile.GetAddressByteSize(), m_objfile.GetSectionList());
     range.SetByteSize (range_len);
 
+    addr_t lsda_data_file_address = LLDB_INVALID_ADDRESS;
+
     if (cie->augmentation[0] == 'z')
     {
         uint32_t aug_data_len = (uint32_t)m_cfi_data.GetULEB128(&offset);
+        if (aug_data_len != 0 && cie->lsda_addr_encoding != DW_EH_PE_omit)
+        {
+            offset_t saved_offset = offset;
+            lsda_data_file_address = m_cfi_data.GetGNUEHPointer(&offset, cie->lsda_addr_encoding, pc_rel_addr, text_addr, data_addr);
+            if (offset - saved_offset != aug_data_len)
+            {
+                // There is more in the augmentation region than we know how to process;
+                // don't read anything.
+                lsda_data_file_address = LLDB_INVALID_ADDRESS;
+            }
+            offset = saved_offset;
+        }
         offset += aug_data_len;
     }
+    Address lsda_data;
+    Address personality_function_ptr;
+
+    if (lsda_data_file_address != LLDB_INVALID_ADDRESS && cie->personality_loc != LLDB_INVALID_ADDRESS)
+    {
+        m_objfile.GetModule()->ResolveFileAddress (lsda_data_file_address, lsda_data);
+        m_objfile.GetModule()->ResolveFileAddress (cie->personality_loc, personality_function_ptr);
+    }
+
+    if (lsda_data.IsValid() && personality_function_ptr.IsValid())
+    {
+        unwind_plan.SetLSDAAddress (lsda_data);
+        unwind_plan.SetPersonalityFunctionPtr (personality_function_ptr);
+    }
 
     uint32_t reg_num = 0;
     int32_t op_offset = 0;
