@@ -468,6 +468,27 @@ SymbolVendor::ClearSymtab()
     }
 }
 
+void
+SymbolVendor::SectionFileAddressesChanged ()
+{
+    ModuleSP module_sp(GetModule());
+    if (module_sp)
+    {
+        ObjectFile *module_objfile = module_sp->GetObjectFile ();
+        if (m_sym_file_ap.get())
+        {
+            ObjectFile *symfile_objfile = m_sym_file_ap->GetObjectFile ();
+            if (symfile_objfile != module_objfile)
+                symfile_objfile->SectionFileAddressesChanged ();
+        }
+        Symtab *symtab = GetSymtab ();
+        if (symtab)
+        {
+            symtab->SectionFileAddressesChanged ();
+        }
+    }
+}
+
 //------------------------------------------------------------------
 // PluginInterface protocol
 //------------------------------------------------------------------
