@@ -269,7 +269,7 @@ StackFrameList::GetFramesUpTo(uint32_t end_idx)
         StreamFile s(stdout, false);
 #endif
         // If we are hiding some frames from the outside world, we need to add those onto the total count of
-        // frames to fetch.  However, we don't need ot do that if end_idx is 0 since in that case we always
+        // frames to fetch.  However, we don't need to do that if end_idx is 0 since in that case we always
         // get the first concrete frame and all the inlined frames below it...  And of course, if end_idx is
         // UINT32_MAX that means get all, so just do that...
         
@@ -288,8 +288,8 @@ StackFrameList::GetFramesUpTo(uint32_t end_idx)
         do
         {
             uint32_t idx = m_concrete_frames_fetched++;
-            lldb::addr_t pc;
-            lldb::addr_t cfa;
+            lldb::addr_t pc = LLDB_INVALID_ADDRESS;
+            lldb::addr_t cfa = LLDB_INVALID_ADDRESS;
             if (idx == 0)
             {
                 // We might have already created frame zero, only create it
@@ -301,7 +301,7 @@ StackFrameList::GetFramesUpTo(uint32_t end_idx)
                     if (reg_ctx_sp)
                     {
 
-                        const bool success = unwinder->GetFrameInfoAtIndex(idx, cfa, pc);
+                        const bool success = unwinder && unwinder->GetFrameInfoAtIndex(idx, cfa, pc);
                         // There shouldn't be any way not to get the frame info for frame 0.
                         // But if the unwinder can't make one, lets make one by hand with the
                         // SP as the CFA and see if that gets any further.
@@ -329,7 +329,7 @@ StackFrameList::GetFramesUpTo(uint32_t end_idx)
             }
             else
             {
-                const bool success = unwinder->GetFrameInfoAtIndex(idx, cfa, pc);
+                const bool success = unwinder && unwinder->GetFrameInfoAtIndex(idx, cfa, pc);
                 if (!success)
                 {
                     // We've gotten to the end of the stack.
@@ -451,14 +451,17 @@ StackFrameList::GetFramesUpTo(uint32_t end_idx)
     {
         if (end_idx < m_concrete_frames_fetched)
             return;
-            
-        uint32_t num_frames = unwinder->GetFramesUpTo(end_idx);
-        if (num_frames <= end_idx + 1)
+
+        if (unwinder)
         {
-            //Done unwinding.
-            m_concrete_frames_fetched = UINT32_MAX;
+            uint32_t num_frames = unwinder->GetFramesUpTo(end_idx);
+            if (num_frames <= end_idx + 1)
+            {
+                //Done unwinding.
+                m_concrete_frames_fetched = UINT32_MAX;
+            }
+            m_frames.resize(num_frames);
         }
-        m_frames.resize(num_frames);
     }
 }
 
@@ -488,7 +491,7 @@ StackFrameList::Dump (Stream *s)
     for (pos = begin; pos != end; ++pos)
     {
         StackFrame *frame = (*pos).get();
-        s->Printf("%p: ", frame);
+        s->Printf("%p: ", static_cast<void*>(frame));
         if (frame)
         {
             frame->GetStackID().Dump (s);
@@ -622,11 +625,14 @@ StackFrameList::GetFrameWithStackID (const StackID &stack_id)
         if (begin != end)
         {
             collection::const_iterator pos = std::lower_bound (begin, end, stack_id, CompareStackID);
-            if (pos != end && (*pos)->GetStackID() == stack_id)
-                return *pos;
+            if (pos != end)
+            {
+                if ((*pos)->GetStackID() == stack_id)
+                    return *pos;
+            }
             
-            if (m_frames.back()->GetStackID() < stack_id)
-                frame_idx = m_frames.size();
+//            if (m_frames.back()->GetStackID() < stack_id)
+//                frame_idx = m_frames.size();
         }
         do
         {
