@@ -21,6 +21,7 @@
 #include "lldb/Core/State.h"
 #include "lldb/Core/Timer.h"
 #include "lldb/Host/Host.h"
+#include "lldb/Host/HostInfo.h"
 #include "lldb/Interpreter/CommandInterpreter.h"
 #include "lldb/Interpreter/OptionGroupPlatform.h"
 #include "lldb/Symbol/ObjectFile.h"
@@ -28,6 +29,8 @@
 #include "lldb/Target/Process.h"
 #include "lldb/Target/TargetList.h"
 
+#include "llvm/ADT/SmallString.h"
+
 using namespace lldb;
 using namespace lldb_private;
 
@@ -66,6 +69,41 @@ TargetList::CreateTarget (Debugger &debugger,
                           bool get_dependent_files,
                           const OptionGroupPlatform *platform_options,
                           TargetSP &target_sp)
+{
+    return CreateTargetInternal (debugger,
+                                 user_exe_path,
+                                 triple_cstr,
+                                 get_dependent_files,
+                                 platform_options,
+                                 target_sp,
+                                 false);
+}
+
+Error
+TargetList::CreateTarget (Debugger &debugger,
+                          const char *user_exe_path,
+                          const ArchSpec& specified_arch,
+                          bool get_dependent_files,
+                          PlatformSP &platform_sp,
+                          TargetSP &target_sp)
+{
+    return CreateTargetInternal (debugger,
+                                 user_exe_path,
+                                 specified_arch,
+                                 get_dependent_files,
+                                 platform_sp,
+                                 target_sp,
+                                 false);
+}
+
+Error
+TargetList::CreateTargetInternal (Debugger &debugger,
+                          const char *user_exe_path,
+                          const char *triple_cstr,
+                          bool get_dependent_files,
+                          const OptionGroupPlatform *platform_options,
+                          TargetSP &target_sp,
+                          bool is_dummy_target)
 {
     Error error;
     PlatformSP platform_sp;
@@ -87,11 +125,29 @@ TargetList::CreateTarget (Debugger &debugger,
 
     bool prefer_platform_arch = false;
     
+    CommandInterpreter &interpreter = debugger.GetCommandInterpreter();
+    if (platform_options && platform_options->PlatformWasSpecified ())
+    {
+        const bool select_platform = true;
+        platform_sp = platform_options->CreatePlatformWithOptions (interpreter,
+                                                                   arch,
+                                                                   select_platform,
+                                                                   error,
+                                                                   platform_arch);
+        if (!platform_sp)
+            return error;
+    }
+    
     if (user_exe_path && user_exe_path[0])
     {
         ModuleSpecList module_specs;
         ModuleSpec module_spec;
         module_spec.GetFileSpec().SetFile(user_exe_path, true);
+        
+        // Resolve the executable in case we are given a path to a application bundle
+        // like a .app bundle on MacOSX
+        Host::ResolveExecutableInBundle (module_spec.GetFileSpec());
+
         lldb::offset_t file_offset = 0;
         lldb::offset_t file_size = 0;
         const size_t num_specs = ObjectFile::GetModuleSpecifications (module_spec.GetFileSpec(), file_offset, file_size, module_specs);
@@ -143,26 +199,92 @@ TargetList::CreateTarget (Debugger &debugger,
                         platform_arch = matching_module_spec.GetArchitecture();
                     }
                 }
+                else
+                {
+                    // No architecture specified, check if there is only one platform for
+                    // all of the architectures.
+                    
+                    typedef std::vector<PlatformSP> PlatformList;
+                    PlatformList platforms;
+                    PlatformSP host_platform_sp = Platform::GetHostPlatform();
+                    for (size_t i=0; i<num_specs; ++i)
+                    {
+                        ModuleSpec module_spec;
+                        if (module_specs.GetModuleSpecAtIndex(i, module_spec))
+                        {
+                            // See if there was a selected platform and check that first
+                            // since the user may have specified it.
+                            if (platform_sp)
+                            {
+                                if (platform_sp->IsCompatibleArchitecture(module_spec.GetArchitecture(), false, NULL))
+                                {
+                                    platforms.push_back(platform_sp);
+                                    continue;
+                                }
+                            }
+                            
+                            // Next check the host platform it if wasn't already checked above
+                            if (host_platform_sp && (!platform_sp || host_platform_sp->GetName() != platform_sp->GetName()))
+                            {
+                                if (host_platform_sp->IsCompatibleArchitecture(module_spec.GetArchitecture(), false, NULL))
+                                {
+                                    platforms.push_back(host_platform_sp);
+                                    continue;
+                                }
+                            }
+                            
+                            // Just find a platform that matches the architecture in the executable file
+                            platforms.push_back(Platform::GetPlatformForArchitecture(module_spec.GetArchitecture(), nullptr));
+                        }
+                    }
+                    
+                    Platform *platform_ptr = NULL;
+                    for (const auto &the_platform_sp : platforms)
+                    {
+                        if (platform_ptr)
+                        {
+                            if (platform_ptr->GetName() != the_platform_sp->GetName())
+                            {
+                                platform_ptr = NULL;
+                                break;
+                            }
+                        }
+                        else
+                        {
+                            platform_ptr = the_platform_sp.get();
+                        }
+                    }
+                    
+                    if (platform_ptr)
+                    {
+                        // All platforms for all modules in the exectuable match, so we can select this platform
+                        platform_sp = platforms.front();
+                    }
+                    else
+                    {
+                        // More than one platform claims to support this file, so the --platform option must be specified
+                        StreamString error_strm;
+                        std::set<Platform *> platform_set;
+                        error_strm.Printf ("more than one platform supports this executable (");
+                        for (const auto &the_platform_sp : platforms)
+                        {
+                            if (platform_set.find(the_platform_sp.get()) == platform_set.end())
+                            {
+                                if (!platform_set.empty())
+                                    error_strm.PutCString(", ");
+                                error_strm.PutCString(the_platform_sp->GetName().GetCString());
+                                platform_set.insert(the_platform_sp.get());
+                            }
+                        }
+                        error_strm.Printf("), use the --platform option to specify a platform");
+                        error.SetErrorString(error_strm.GetString().c_str());
+                        return error;
+                    }
+                }
             }
         }
     }
 
-    CommandInterpreter &interpreter = debugger.GetCommandInterpreter();
-    if (platform_options)
-    {
-        if (platform_options->PlatformWasSpecified ())
-        {
-            const bool select_platform = true;
-            platform_sp = platform_options->CreatePlatformWithOptions (interpreter,
-                                                                       arch,
-                                                                       select_platform, 
-                                                                       error,
-                                                                       platform_arch);
-            if (!platform_sp)
-                return error;
-        }
-    }
-    
     if (!platform_sp)
     {
         // Get the current platform and make sure it is compatible with the
@@ -172,7 +294,11 @@ TargetList::CreateTarget (Debugger &debugger,
         if (!prefer_platform_arch && arch.IsValid())
         {
             if (!platform_sp->IsCompatibleArchitecture(arch, false, &platform_arch))
+            {
                 platform_sp = Platform::GetPlatformForArchitecture(arch, &platform_arch);
+                if (platform_sp)
+                    debugger.GetPlatformList().SetSelectedPlatform(platform_sp);
+            }
         }
         else if (platform_arch.IsValid())
         {
@@ -180,30 +306,69 @@ TargetList::CreateTarget (Debugger &debugger,
             // a single architecture which should be used
             ArchSpec fixed_platform_arch;
             if (!platform_sp->IsCompatibleArchitecture(platform_arch, false, &fixed_platform_arch))
+            {
                 platform_sp = Platform::GetPlatformForArchitecture(platform_arch, &fixed_platform_arch);
+                if (platform_sp)
+                    debugger.GetPlatformList().SetSelectedPlatform(platform_sp);
+            }
         }
     }
     
     if (!platform_arch.IsValid())
         platform_arch = arch;
 
-    error = TargetList::CreateTarget (debugger,
-                                      user_exe_path,
-                                      platform_arch,
-                                      get_dependent_files,
-                                      platform_sp,
-                                      target_sp);
+    error = TargetList::CreateTargetInternal (debugger,
+                                              user_exe_path,
+                                              platform_arch,
+                                              get_dependent_files,
+                                              platform_sp,
+                                              target_sp,
+                                              is_dummy_target);
     return error;
 }
 
-Error
-TargetList::CreateTarget (Debugger &debugger,
-                          const char *user_exe_path,
-                          const ArchSpec& specified_arch,
-                          bool get_dependent_files,
-                          PlatformSP &platform_sp,
-                          TargetSP &target_sp)
+lldb::TargetSP
+TargetList::GetDummyTarget (lldb_private::Debugger &debugger)
 {
+    // FIXME: Maybe the dummy target should be per-Debugger
+    if (!m_dummy_target_sp || !m_dummy_target_sp->IsValid())
+    {
+        ArchSpec arch(Target::GetDefaultArchitecture());
+        if (!arch.IsValid())
+            arch = HostInfo::GetArchitecture();
+        Error err = CreateDummyTarget(debugger,
+                                      arch.GetTriple().getTriple().c_str(),
+                                      m_dummy_target_sp);
+    }
+
+    return m_dummy_target_sp;
+}
+
+Error
+TargetList::CreateDummyTarget (Debugger &debugger,
+                               const char *specified_arch_name,
+                               lldb::TargetSP &target_sp)
+{
+    PlatformSP host_platform_sp(Platform::GetHostPlatform());
+    return CreateTargetInternal (debugger,
+                                 (const char *) nullptr,
+                                 specified_arch_name,
+                                 false,
+                                 (const OptionGroupPlatform *) nullptr,
+                                 target_sp,
+                                 true);
+}
+
+Error
+TargetList::CreateTargetInternal (Debugger &debugger,
+                      const char *user_exe_path,
+                      const ArchSpec& specified_arch,
+                      bool get_dependent_files,
+                      lldb::PlatformSP &platform_sp,
+                      lldb::TargetSP &target_sp,
+                      bool is_dummy_target)
+{
+
     Timer scoped_timer (__PRETTY_FUNCTION__,
                         "TargetList::CreateTarget (file = '%s', arch = '%s')",
                         user_exe_path,
@@ -212,17 +377,10 @@ TargetList::CreateTarget (Debugger &debugger,
 
     ArchSpec arch(specified_arch);
 
-    if (platform_sp)
+    if (arch.IsValid())
     {
-        if (arch.IsValid())
-        {
-            if (!platform_sp->IsCompatibleArchitecture(arch, false, NULL))
-                platform_sp = Platform::GetPlatformForArchitecture(specified_arch, &arch);
-        }
-    }
-    else if (arch.IsValid())
-    {
-        platform_sp = Platform::GetPlatformForArchitecture(specified_arch, &arch);
+        if (!platform_sp || !platform_sp->IsCompatibleArchitecture(arch, false, NULL))
+            platform_sp = Platform::GetPlatformForArchitecture(specified_arch, &arch);
     }
     
     if (!platform_sp)
@@ -236,15 +394,13 @@ TargetList::CreateTarget (Debugger &debugger,
     {
         // we want to expand the tilde but we don't want to resolve any symbolic links
         // so we can't use the FileSpec constructor's resolve flag
-        char unglobbed_path[PATH_MAX];
-        unglobbed_path[0] = '\0';
+        llvm::SmallString<64> unglobbed_path(user_exe_path);
+        FileSpec::ResolveUsername(unglobbed_path);
 
-        size_t return_count = FileSpec::ResolveUsername(user_exe_path, unglobbed_path, sizeof(unglobbed_path));
-
-        if (return_count == 0 || return_count >= sizeof(unglobbed_path))
-            ::snprintf (unglobbed_path, sizeof(unglobbed_path), "%s", user_exe_path);
-
-        file = FileSpec(unglobbed_path, false);
+        if (unglobbed_path.empty())
+            file = FileSpec(user_exe_path, false);
+        else
+            file = FileSpec(unglobbed_path.c_str(), false);
     }
 
     bool user_exe_path_is_bundle = false;
@@ -255,7 +411,7 @@ TargetList::CreateTarget (Debugger &debugger,
         if (file.GetFileType() == FileSpec::eFileTypeDirectory)
             user_exe_path_is_bundle = true;
 
-        if (file.IsRelativeToCurrentWorkingDirectory())
+        if (file.IsRelativeToCurrentWorkingDirectory() && user_exe_path)
         {
             // Ignore paths that start with "./" and "../"
             if (!((user_exe_path[0] == '.' && user_exe_path[1] == '/') ||
@@ -278,8 +434,8 @@ TargetList::CreateTarget (Debugger &debugger,
         if (platform_sp)
         {
             FileSpecList executable_search_paths (Target::GetDefaultExecutableSearchPaths());
-            error = platform_sp->ResolveExecutable (file,
-                                                    arch,
+            ModuleSpec module_spec(file, arch);
+            error = platform_sp->ResolveExecutable (module_spec,
                                                     exe_module_sp, 
                                                     executable_search_paths.GetSize() ? &executable_search_paths : NULL);
         }
@@ -301,7 +457,7 @@ TargetList::CreateTarget (Debugger &debugger,
                 }
                 return error;
             }
-            target_sp.reset(new Target(debugger, arch, platform_sp));
+            target_sp.reset(new Target(debugger, arch, platform_sp, is_dummy_target));
             target_sp->SetExecutableModule (exe_module_sp, get_dependent_files);
             if (user_exe_path_is_bundle)
                 exe_module_sp->GetFileSpec().GetPath(resolved_bundle_exe_path, sizeof(resolved_bundle_exe_path));
@@ -311,7 +467,7 @@ TargetList::CreateTarget (Debugger &debugger,
     {
         // No file was specified, just create an empty target with any arch
         // if a valid arch was specified
-        target_sp.reset(new Target(debugger, arch, platform_sp));
+        target_sp.reset(new Target(debugger, arch, platform_sp, is_dummy_target));
     }
 
     if (target_sp)
@@ -338,11 +494,20 @@ TargetList::CreateTarget (Debugger &debugger,
             file_dir.GetDirectory() = file.GetDirectory();
             target_sp->GetExecutableSearchPaths ().Append (file_dir);
         }
-        Mutex::Locker locker(m_target_list_mutex);
-        m_selected_target_idx = m_target_list.size();
-        m_target_list.push_back(target_sp);
-        
-        
+
+        // Don't put the dummy target in the target list, it's held separately.
+        if (!is_dummy_target)
+        {
+            Mutex::Locker locker(m_target_list_mutex);
+            m_selected_target_idx = m_target_list.size();
+            m_target_list.push_back(target_sp);
+            // Now prime this from the dummy target:
+            target_sp->PrimeFromDummyTarget(debugger.GetDummyTarget());
+        }
+        else
+        {
+            m_dummy_target_sp = target_sp;
+        }
     }
 
     return error;
