@@ -5,6 +5,7 @@ Test lldb 'process connect' command.
 import os
 import unittest2
 import lldb
+import re
 from lldbtest import *
 
 class ConnectRemoteTestCase(TestBase):
@@ -12,7 +13,7 @@ class ConnectRemoteTestCase(TestBase):
     mydir = TestBase.compute_mydir(__file__)
 
     def test_connect_remote(self):
-        """Test "process connect connect:://localhost:12345"."""
+        """Test "process connect connect:://localhost:[port]"."""
 
         import pexpect
         # First, we'll start a fake debugserver (a simple echo server).
@@ -28,10 +29,13 @@ class ConnectRemoteTestCase(TestBase):
         self.addTearDownHook(shutdown_fakeserver)
 
         # Wait until we receive the server ready message before continuing.
-        fakeserver.expect_exact('Listening on localhost:12345')
+        line = fakeserver.readline()
+        self.assertTrue(line.startswith("Listening on localhost:"))
+        port = int(re.match('Listening on localhost:([0-9]+)', line).group(1))
+        self.assertTrue(port > 0)
 
         # Connect to the fake server....
-        self.runCmd("process connect -p gdb-remote connect://localhost:12345")
+        self.runCmd("process connect -p gdb-remote connect://localhost:" + str(port))
 
 
 if __name__ == '__main__':
