@@ -14,12 +14,20 @@ class StdVBoolDataFormatterTestCase(TestBase):
 
     @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
     @dsym_test
+    @skipIfDarwin
     def test_with_dsym_and_run_command(self):
         """Test data formatter commands."""
         self.buildDsym()
         self.data_formatter_commands()
 
+    @expectedFailureFreeBSD("llvm.org/pr20548") # fails to build on lab.llvm.org buildbot
+    @expectedFailureLinux # non-core functionality, need to reenable and fix
+                          # later (DES 2014.11.07). Most likely failing because
+                          # of mis-match is version of libstdc++ supported by
+                          # the data-formatters.
     @dwarf_test
+    @skipIfWindows # http://llvm.org/pr21800
+    @skipIfDarwin
     def test_with_dwarf_and_run_command(self):
         """Test data formatter commands."""
         self.buildDwarf()
