@@ -14,14 +14,17 @@ class StdVectorDataFormatterTestCase(TestBase):
 
     @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
     @dsym_test
-    @expectedFailureDarwin("llvm.org/pr20264")
     def test_with_dsym_and_run_command(self):
         """Test data formatter commands."""
         self.buildDsym()
         self.data_formatter_commands()
 
     @dwarf_test
-    @expectedFailureClang # llvm.org/pr15301 LLDB prints incorrect sizes of STL containers
+    @skipIfFreeBSD
+    @expectedFailureLinux # non-core functionality, need to reenable and fix
+                          # later (DES 2014.11.07). Most likely failing because
+                          # of mis-match is version of libstdc++ supported by
+                          # the data-formatters.
     @expectedFailureIcc # llvm.org/pr15301 LLDB prints incorrect sizes of STL containers
     @expectedFailureGcc # llvm.org/pr17499 The data formatter cannot parse STL containers
     def test_with_dwarf_and_run_command(self):
