@@ -20,8 +20,6 @@ class StaticVariableTestCase(TestBase):
         self.buildDsym()
         self.static_variable_commands()
 
-    @expectedFailureFreeBSD('llvm.org/pr15261', failing_compilers) # lldb on FreeBSD does not display the size of (class or file)static arrays
-    @expectedFailureLinux('llvm.org/pr15261', failing_compilers) # lldb on Linux does not display the size of (class or file)static arrays
     @dwarf_test
     def test_with_dwarf_and_run_command(self):
         """Test that file and class static variables display correctly."""
@@ -39,6 +37,8 @@ class StaticVariableTestCase(TestBase):
         self.static_variable_python()
 
     @expectedFailureDarwin(9980907)
+    @expectedFailureClang('Clang emits incomplete debug info.')
+    @expectedFailureGcc('GCC emits incomplete debug info.')
     @python_api_test
     @dwarf_test
     def test_with_dwarf_and_python_api(self):
@@ -67,13 +67,13 @@ class StaticVariableTestCase(TestBase):
 
         # global variables are no longer displayed with the "frame variable" command. 
         self.expect('target variable A::g_points', VARIABLES_DISPLAYED_CORRECTLY,
-            substrs = ['(PointType [2]) A::g_points'])
+            patterns=['\(PointType \[[1-9]*\]\) A::g_points = {.*}'])
         self.expect('target variable g_points', VARIABLES_DISPLAYED_CORRECTLY,
             substrs = ['(PointType [2]) g_points'])
 
         # On Mac OS X, gcc 4.2 emits the wrong debug info for A::g_points.
         # A::g_points is an array of two elements.
-        if sys.platform.startswith("darwin") and self.getCompiler() in ['clang', 'llvm-gcc']:
+        if sys.platform.startswith("darwin") or sys.platform.startswith("linux"):
             self.expect("target variable A::g_points[1].x", VARIABLES_DISPLAYED_CORRECTLY,
                 startstr = "(int) A::g_points[1].x = 11")
 
@@ -114,8 +114,7 @@ class StaticVariableTestCase(TestBase):
             if name == 'g_points':
                 self.assertTrue(val.GetValueType() == lldb.eValueTypeVariableStatic)
                 self.assertTrue(val.GetNumChildren() == 2)
-            elif name == 'A::g_points' and self.getCompiler() in ['clang', 'llvm-gcc']:
-                # On Mac OS X, gcc 4.2 emits the wrong debug info for A::g_points.        
+            elif name == 'A::g_points':
                 self.assertTrue(val.GetValueType() == lldb.eValueTypeVariableGlobal)
                 self.assertTrue(val.GetNumChildren() == 2)
                 child1 = val.GetChildAtIndex(1)
