@@ -24,5 +24,7 @@ int main (int argc, char const *argv[])
     Foo<int> foo_x('a');
     Foo<wchar_t> foo_y(L'a');
     const wchar_t *mazeltov = L"מזל טוב";
+    wchar_t *ws_NULL = nullptr;
+    wchar_t *ws_empty = L"";
     return 0; // Set break point at this line.
 }
