@@ -15,21 +15,21 @@ class ObjCDynamicSBTypeTestCase(TestBase):
 
     @dsym_test
     @skipIfi386
-    def test_nsimage_dyn_with_dsym(self):
+    def test_dyn_with_dsym(self):
         """Test that we are able to properly report a usable dynamic type."""
         d = {'EXE': self.exe_name}
         self.buildDsym(dictionary=d)
         self.setTearDownCleanup(dictionary=d)
-        self.nsimage_dyn(self.exe_name)
+        self.dyn(self.exe_name)
 
     @dwarf_test
     @skipIfi386
-    def test_nsimage_dyn_with_dwarf(self):
+    def test_dyn_with_dwarf(self):
         """Test that we are able to properly report a usable dynamic type."""
         d = {'EXE': self.exe_name}
         self.buildDwarf(dictionary=d)
         self.setTearDownCleanup(dictionary=d)
-        self.nsimage_dyn(self.exe_name)
+        self.dyn(self.exe_name)
 
     def setUp(self):
         # Call super's setUp().
@@ -40,7 +40,7 @@ class ObjCDynamicSBTypeTestCase(TestBase):
         self.main_source = "main.m"
         self.line = line_number(self.main_source, '// Set breakpoint here.')
 
-    def nsimage_dyn(self, exe_name):
+    def dyn(self, exe_name):
         """Test that we are able to properly report a usable dynamic type."""
         exe = os.path.join(os.getcwd(), exe_name)
         self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
@@ -49,14 +49,6 @@ class ObjCDynamicSBTypeTestCase(TestBase):
 
         self.runCmd("run", RUN_SUCCEEDED)
 
-        image = self.frame().EvaluateExpression("(id)image",lldb.eDynamicCanRunTarget)
-        self.assertTrue(image.GetTypeName() == "NSImage *", "The SBValue is properly type-named")
-        image_type = image.GetType()
-        self.assertTrue(image_type.GetName() == "NSImage *", "The dynamic SBType is for the correct type")
-        image_pointee_type = image_type.GetPointeeType()
-        self.assertTrue(image_pointee_type.GetName() == "NSImage", "The dynamic type figures out its pointee type just fine")
-        self.assertTrue(image_pointee_type.GetDirectBaseClassAtIndex(0).GetName() == "NSObject", "The dynamic type can go back to its base class")
-
         v_object = self.frame().FindVariable("object").GetDynamicValue(lldb.eDynamicCanRunTarget)
         v_base = self.frame().FindVariable("base").GetDynamicValue(lldb.eDynamicCanRunTarget)
         self.assertTrue(v_object.GetTypeName() == "MyDerivedClass *", "The NSObject is properly type-named")
