@@ -13,6 +13,7 @@ the initial set of tests implemented.
 import gdbremote_testcase
 import lldbgdbserverutils
 import platform
+import signal
 import unittest2
 from lldbtest import *
 
@@ -126,7 +127,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.add_no_ack_remote_stream()
         self.add_verified_launch_packets(launch_args)
         self.test_sequence.add_log_lines(
-            ["read packet: $vCont;c#00",
+            ["read packet: $vCont;c#a8",
              "send packet: $W00#00"],
             True)
 
@@ -158,7 +159,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.add_no_ack_remote_stream()
         self.add_verified_launch_packets(launch_args)
         self.test_sequence.add_log_lines(
-            ["read packet: $vCont;c#00",
+            ["read packet: $vCont;c#a8",
              "send packet: $W{0:02x}#00".format(RETVAL)],
             True)
 
@@ -188,7 +189,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.add_no_ack_remote_stream()
         self.add_verified_launch_packets(launch_args)
         self.test_sequence.add_log_lines(
-            ["read packet: $c#00",
+            ["read packet: $c#63",
              "send packet: $W00#00"],
             True)
 
@@ -218,7 +219,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.add_no_ack_remote_stream()
         self.add_verified_launch_packets(launch_args)
         self.test_sequence.add_log_lines(
-            ["read packet: $vCont;c#00",
+            ["read packet: $vCont;c#a8",
              {"type":"output_match", "regex":r"^hello, world\r\n$" },
              "send packet: $W00#00"],
             True)
@@ -271,111 +272,10 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.buildDwarf()
         self.first_launch_stop_reply_thread_matches_first_qC()
 
-    def qProcessInfo_returns_running_process(self):
-        procs = self.prep_debug_monitor_and_inferior()
-        self.add_process_info_collection_packets()
-
-        # Run the stream
-        context = self.expect_gdbremote_sequence()
-        self.assertIsNotNone(context)
-
-        # Gather process info response
-        process_info = self.parse_process_info_response(context)
-        self.assertIsNotNone(process_info)
-
-        # Ensure the process id looks reasonable.
-        pid_text = process_info.get("pid")
-        self.assertIsNotNone(pid_text)
-        pid = int(pid_text, base=16)
-        self.assertNotEqual(0, pid)
-
-        # If possible, verify that the process is running.
-        self.assertTrue(lldbgdbserverutils.process_is_running(pid, True))
-
-    @debugserver_test
-    @dsym_test
-    def test_qProcessInfo_returns_running_process_debugserver_dsym(self):
-        self.init_debugserver_test()
-        self.buildDsym()
-        self.qProcessInfo_returns_running_process()
-
-    @llgs_test
-    @dwarf_test
-    def test_qProcessInfo_returns_running_process_llgs_dwarf(self):
-        self.init_llgs_test()
-        self.buildDwarf()
-        self.qProcessInfo_returns_running_process()
-
-    def attach_commandline_qProcessInfo_reports_correct_pid(self):
-        procs = self.prep_debug_monitor_and_inferior()
-        self.assertIsNotNone(procs)
-        self.add_process_info_collection_packets()
-
-        # Run the stream
-        context = self.expect_gdbremote_sequence()
-        self.assertIsNotNone(context)
-
-        # Gather process info response
-        process_info = self.parse_process_info_response(context)
-        self.assertIsNotNone(process_info)
-
-        # Ensure the process id matches what we expected.
-        pid_text = process_info.get('pid', None)
-        self.assertIsNotNone(pid_text)
-        reported_pid = int(pid_text, base=16)
-        self.assertEqual(reported_pid, procs["inferior"].pid)
-
-    @debugserver_test
-    @dsym_test
-    def test_attach_commandline_qProcessInfo_reports_correct_pid_debugserver_dsym(self):
-        self.init_debugserver_test()
-        self.buildDsym()
-        self.set_inferior_startup_attach()
-        self.attach_commandline_qProcessInfo_reports_correct_pid()
-
-    @llgs_test
-    @dwarf_test
-    def test_attach_commandline_qProcessInfo_reports_correct_pid_llgs_dwarf(self):
-        self.init_llgs_test()
-        self.buildDwarf()
-        self.set_inferior_startup_attach()
-        self.attach_commandline_qProcessInfo_reports_correct_pid()
-
-    def qProcessInfo_reports_valid_endian(self):
-        procs = self.prep_debug_monitor_and_inferior()
-        self.add_process_info_collection_packets()
-
-        # Run the stream
-        context = self.expect_gdbremote_sequence()
-        self.assertIsNotNone(context)
-
-        # Gather process info response
-        process_info = self.parse_process_info_response(context)
-        self.assertIsNotNone(process_info)
-
-        # Ensure the process id looks reasonable.
-        endian = process_info.get("endian")
-        self.assertIsNotNone(endian)
-        self.assertTrue(endian in ["little", "big", "pdp"])
-
-    @debugserver_test
-    @dsym_test
-    def test_qProcessInfo_reports_valid_endian_debugserver_dsym(self):
-        self.init_debugserver_test()
-        self.buildDsym()
-        self.qProcessInfo_reports_valid_endian()
-
-    @llgs_test
-    @dwarf_test
-    def test_qProcessInfo_reports_valid_endian_llgs_dwarf(self):
-        self.init_llgs_test()
-        self.buildDwarf()
-        self.qProcessInfo_reports_valid_endian()
-
     def attach_commandline_continue_app_exits(self):
         procs = self.prep_debug_monitor_and_inferior()
         self.test_sequence.add_log_lines(
-            ["read packet: $vCont;c#00",
+            ["read packet: $vCont;c#a8",
              "send packet: $W00#00"],
             True)
         self.expect_gdbremote_sequence()
@@ -837,7 +737,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.set_inferior_startup_attach()
         self.Hg_switches_to_3_threads()
 
-    def Hc_then_Csignal_signals_correct_thread(self):
+    def Hc_then_Csignal_signals_correct_thread(self, segfault_signo):
         # NOTE only run this one in inferior-launched mode: we can't grab inferior stdout when running attached,
         # and the test requires getting stdout from the exe.
 
@@ -855,7 +755,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
 
         # Launch/attach.  (In our case, this should only ever be launched since we need inferior stdout/stderr).
         procs = self.prep_debug_monitor_and_inferior(inferior_args=inferior_args)
-        self.test_sequence.add_log_lines(["read packet: $c#00"], True)
+        self.test_sequence.add_log_lines(["read packet: $c#63"], True)
         context = self.expect_gdbremote_sequence()
 
         # Let the inferior process have a few moments to start up the thread when launched.
@@ -879,7 +779,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
             context = self.expect_gdbremote_sequence(timeout_seconds=10)
             self.assertIsNotNone(context)
             signo = context.get("signo")
-            self.assertEqual(int(signo, 16), self.TARGET_EXC_BAD_ACCESS)
+            self.assertEqual(int(signo, 16), segfault_signo)
 
             # Ensure we haven't seen this tid yet.
             thread_id = int(context.get("thread_id"), 16)
@@ -905,7 +805,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
                  # Need to rectify behavior here.  The linux behavior is more intuitive to me since we're essentially swapping out
                  # an about-to-be-delivered signal (for which we already sent a stop packet) to a different signal.
                  # {"direction":"send", "regex":r"^\$T([0-9a-fA-F]{2})thread:([0-9a-fA-F]+);", "capture":{1:"stop_signo", 2:"stop_thread_id"} },
-                 #  "read packet: $c#00",
+                 #  "read packet: $c#63",
                  { "type":"output_match", "regex":r"^received SIGUSR1 on thread id: ([0-9a-fA-F]+)\r\nthread ([0-9a-fA-F]+): past SIGSEGV\r\n", "capture":{ 1:"print_thread_id", 2:"post_handle_thread_id" } },
                 ],
                 True)
@@ -946,7 +846,8 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.init_debugserver_test()
         self.buildDsym()
         self.set_inferior_startup_launch()
-        self.Hc_then_Csignal_signals_correct_thread()
+        # Darwin debugserver translates some signals like SIGSEGV into some gdb expectations about fixed signal numbers.
+        self.Hc_then_Csignal_signals_correct_thread(self.TARGET_EXC_BAD_ACCESS)
 
     @llgs_test
     @dwarf_test
@@ -954,7 +855,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.init_llgs_test()
         self.buildDwarf()
         self.set_inferior_startup_launch()
-        self.Hc_then_Csignal_signals_correct_thread()
+        self.Hc_then_Csignal_signals_correct_thread(signal.SIGSEGV)
 
     def m_packet_reads_memory(self):
         # This is the memory we will write into the inferior and then ensure we can read back with $m.
@@ -968,7 +869,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.test_sequence.add_log_lines(
             [
              # Start running after initial stop.
-             "read packet: $c#00",
+             "read packet: $c#63",
              # Match output line that prints the memory address of the message buffer within the inferior. 
              # Note we require launch-only testing so we can get inferior otuput.
              { "type":"output_match", "regex":r"^data address: 0x([0-9a-fA-F]+)\r\n$", "capture":{ 1:"message_address"} },
@@ -1054,7 +955,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.test_sequence.add_log_lines(
             [
              # Start running after initial stop.
-             "read packet: $c#00",
+             "read packet: $c#63",
              # Match output line that prints the memory address of the message buffer within the inferior. 
              # Note we require launch-only testing so we can get inferior otuput.
              { "type":"output_match", "regex":r"^code address: 0x([0-9a-fA-F]+)\r\n$", "capture":{ 1:"code_address"} },
@@ -1117,7 +1018,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.test_sequence.add_log_lines(
             [
              # Start running after initial stop.
-             "read packet: $c#00",
+             "read packet: $c#63",
              # Match output line that prints the memory address of the message buffer within the inferior. 
              # Note we require launch-only testing so we can get inferior otuput.
              { "type":"output_match", "regex":r"^stack address: 0x([0-9a-fA-F]+)\r\n$", "capture":{ 1:"stack_address"} },
@@ -1180,7 +1081,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.test_sequence.add_log_lines(
             [
              # Start running after initial stop.
-             "read packet: $c#00",
+             "read packet: $c#63",
              # Match output line that prints the memory address of the message buffer within the inferior. 
              # Note we require launch-only testing so we can get inferior otuput.
              { "type":"output_match", "regex":r"^heap address: 0x([0-9a-fA-F]+)\r\n$", "capture":{ 1:"heap_address"} },
@@ -1245,7 +1146,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.add_process_info_collection_packets()
         self.test_sequence.add_log_lines(
             [# Start running after initial stop.
-             "read packet: $c#00",
+             "read packet: $c#63",
              # Match output line that prints the memory address of the function call entry point.
              # Note we require launch-only testing so we can get inferior otuput.
              { "type":"output_match", "regex":r"^code address: 0x([0-9a-fA-F]+)\r\n$", "capture":{ 1:"function_address"} },
@@ -1325,7 +1226,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
             # Verify the stub could unset it.
             "send packet: $OK#00",
             # Continue running.
-            "read packet: $c#00",
+            "read packet: $c#63",
             # We should now receive the output from the call.
             { "type":"output_match", "regex":r"^hello, world\r\n$" },
             # And wait for program completion.
@@ -1389,7 +1290,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.test_sequence.add_log_lines(
             [
              # Start running after initial stop.
-             "read packet: $c#00",
+             "read packet: $c#63",
              # Match output line that prints the memory address of the message buffer within the inferior. 
              # Note we require launch-only testing so we can get inferior otuput.
              { "type":"output_match", "regex":r"^data address: 0x([0-9a-fA-F]+)\r\n$", "capture":{ 1:"message_address"} },
@@ -1413,7 +1314,7 @@ class LldbGdbServerTestCase(gdbremote_testcase.GdbRemoteTestCaseBase):
         self.test_sequence.add_log_lines(
             ["read packet: $M{0:x},{1:x}:{2}#00".format(message_address, len(hex_encoded_message)/2, hex_encoded_message),
              "send packet: $OK#00",
-             "read packet: $c#00",
+             "read packet: $c#63",
              { "type":"output_match", "regex":r"^message: (.+)\r\n$", "capture":{ 1:"printed_message"} },
              "send packet: $W00#00",
             ], True)
